"""Main app class."""
from typing import cast

from textual.app import App

from recoverpy.ui.css import get_css
from recoverpy.ui.screens.screen_modal import ModalScreen
from recoverpy.ui.screens.screen_params import ParamsScreen
from recoverpy.ui.screens.screen_path_edit import PathEditScreen
from recoverpy.ui.screens.screen_result import ResultScreen
from recoverpy.ui.screens.screen_save import SaveScreen
from recoverpy.ui.screens.screen_search import SearchScreen


class RecoverpyApp(App):  # type: ignore
    SCREENS = {
        "params": ParamsScreen(),
        "search": SearchScreen(),
        "result": ResultScreen(),
        "save": SaveScreen(),
        "path_edit": PathEditScreen(),
        "modal": ModalScreen(),
    }
    CSS_PATH = get_css()  # type: ignore # mypy bug, List[str] = List[Union[str, PurePath]]

    def __init__(self, *args, **kwargs) -> None:  # type: ignore
        super().__init__(*args, **kwargs)
        self._is_user_root = True

    def on_mount(self) -> None:
        if self._is_user_root:
            self.push_screen("params")
        else:
            cast(ModalScreen, self.get_screen("modal")).set(
                message="You must be root to run this app", callback=self.exit
            )
            self.push_screen("modal")

    async def on_params_screen_continue(self, message: ParamsScreen.Continue) -> None:
        self.pop_screen()
        await self.push_screen("search")
        await self.get_screen("search").post_message(
            SearchScreen.Start(
                self, message.searched_string, message.selected_partition
            )
        )

    async def on_search_screen_open(self, message: SearchScreen.Open) -> None:
        await self.push_screen("result")
        cast(ResultScreen, self.get_screen("result")).set(
            message.partition, message.block_size, message.inode
        )

    async def on_save_screen_saved(self, message: SaveScreen.Saved) -> None:
        cast(ModalScreen, self.get_screen("modal")).set(
            f"Saved results to {message.save_path}"
        )
        await self.push_screen("modal")



"""Screen displaying grep results."""

from asyncio import ensure_future, sleep
from typing import List

from textual._types import MessageTarget
from textual.app import ComposeResult
from textual.containers import Horizontal, Vertical
from textual.message import Message
from textual.screen import Screen
from textual.widgets import Button, Label

from recoverpy.lib.search.search_engine import SearchEngine
from recoverpy.ui.widgets.grep_result_list import GrepResultList


class SearchScreen(Screen):
    _grep_result_list: GrepResultList
    _result_count_label: Label
    _progress_title_label: Label
    _progress_label: Label
    _open_button: Button
    search_engine: SearchEngine

    class Start(Message):
        def __init__(
            self, sender: MessageTarget, searched_string: str, selected_partition: str
        ) -> None:
            self.searched_string = searched_string
            self.selected_partition = selected_partition
            super().__init__(sender)

    class Open(Message):
        def __init__(
            self, sender: MessageTarget, inode: int, block_size: int, partition: str
        ) -> None:
            self.inode = inode
            self.block_size = block_size
            self.partition = partition
            super().__init__(sender)

    class InfoContainer(Horizontal):
        def __init__(self, *args, **kwargs) -> None:  # type: ignore
            super().__init__(classes="info-container", *args, **kwargs)

    def __init__(self, *args, **kwargs) -> None:  # type: ignore
        self.results: List[str] = []
        super().__init__(*args, **kwargs)

    def compose(self) -> ComposeResult:
        self._grep_result_list = GrepResultList()
        self._result_count_label = Label("0", id="result-count")
        self._progress_title_label = Label("", id="progress-title")
        self._progress_label = Label("", id="progress")
        self._open_button = Button(label="Open", id="open-button", disabled=True)

        yield self._grep_result_list
        yield Vertical(
            self.InfoContainer(Label("- result count -", id="result-count-title")),
            self.InfoContainer(self._result_count_label),
            self.InfoContainer(self._progress_title_label),
            self.InfoContainer(self._progress_label),
            id="info-bar",
        )
        yield self._open_button
        yield Button("Exit", id="exit-button")

    async def on_search_screen_start(self, message: Start) -> None:
        self.search_engine = SearchEngine(
            message.selected_partition, message.searched_string
        )
        while self._grep_result_list not in self.visible_widgets:
            continue
        await self.search_engine.start_search()
        ensure_future(
            self._grep_result_list.start_consumer(self.search_engine.list_items_queue)
        )
        ensure_future(self.get_progress())

    async def get_progress(self) -> None:
        while True:
            self._result_count_label.update(
                str(self.search_engine.search_progress.result_count)
            )
            if self.search_engine.search_progress.progress_percent != 0.0:
                if self._progress_title_label.renderable == "":
                    self._progress_title_label.update("- progress -")
                self._progress_label.update(
                    f"{self.search_engine.search_progress.progress_percent}%"
                )
            await sleep(0.1)

    async def on_button_pressed(self, event: Button.Pressed) -> None:
        button_id = event.button.id
        if button_id == "exit-button":
            self.search_engine.stop_search()
            self.app.exit()
            exit()
        elif button_id == "open-button":
            await self.app.post_message(
                self.Open(
                    self,
                    self._grep_result_list.grep_results[
                        self._grep_result_list.index
                    ].inode,
                    self.search_engine.search_params.block_size,
                    self.search_engine.search_params.partition,
                )
            )

    async def on_list_view_highlighted(self) -> None:
        self._open_button.disabled = False



from recoverpy.lib.helper import get_block_size


class SearchParams:
    def __init__(self, partition: str, search_string: str):
        self.search_string = search_string
        self.partition = partition
        self.block_size = get_block_size(partition)
        self.searched_lines = search_string.strip().splitlines()
        self.is_multi_line = len(self.searched_lines) > 1



"""Screen used to select ressult save path."""

from textual._types import MessageTarget
from textual.app import ComposeResult
from textual.containers import Horizontal
from textual.events import Event
from textual.message import Message
from textual.screen import Screen
from textual.widgets import Button

from recoverpy.ui.widgets.directory_tree import DirectoryTree


class PathEditScreen(Screen):
    def __init__(self, *args, **kwargs) -> None:  # type: ignore
        self._directory_tree = DirectoryTree("/")
        super().__init__(*args, **kwargs)

    class Confirm(Message):
        def __init__(self, sender: MessageTarget, selected_dir: str) -> None:
            self.selected_dir = selected_dir
            super().__init__(sender)

    def compose(self) -> ComposeResult:
        yield self._directory_tree
        yield Horizontal(
            Button("Confirm", id="confirm-button"), id="path-edit-button-container"
        )

    async def on_button_pressed(self, event: Event) -> None:
        event.stop()
        await self.app.get_screen("save").post_message(
            self.Confirm(self, self._directory_tree.selected_dir)
        )
        self.app.pop_screen()



from typing import Optional

from textual.widgets import Label, ListItem

from recoverpy.lib.helper import get_inode, get_printable


class GrepResult:
    def __init__(self, line: str):
        self.inode = get_inode(line)
        self.line = get_printable(line)
        self.list_item: Optional[ListItem] = None

    def create_list_item(self, css_class: str) -> None:
        try:
            self.list_item = ListItem(
                Label(str(self.line), markup=False), classes=css_class
            )
        except RuntimeError:
            # No running event loop exception during tests in python 3.8
            return



"""Screen displaying dd results."""
from subprocess import CalledProcessError
from typing import Optional, cast

from textual.app import ComposeResult
from textual.containers import Horizontal
from textual.screen import Screen
from textual.widgets import Button, Label, TextLog

from recoverpy.lib.helper import decode_result, get_dd_output, get_printable
from recoverpy.lib.saver import Saver
from recoverpy.ui.screens.screen_save import SaveScreen


class ResultScreen(Screen):
    def __init__(self, *args, **kwargs) -> None:  # type: ignore
        self._saver = Saver()
        self._partition = ""
        self._block_size = 0
        self._inode = 0
        self._inode_label = Label("", id="inode-label")
        self._block_count_label = Label("0 block selected", id="block-count")
        self._block_content = TextLog(markup=False, wrap=True)
        self._raw_block_content: Optional[str] = None
        self._save_button = Button(label="Save", id="save-button", disabled=True)
        super().__init__(*args, **kwargs)

    def set(self, partition: str, block_size: int, inode: int) -> None:
        self._saver.reset()
        self._save_button.disabled = True
        self._partition = partition
        self._block_size = block_size
        self._inode = inode
        self._update()
        self._block_count_label.update("0 block selected")

    def _update(self) -> None:
        self._update_inode_label()
        try:
            self._update_block_content()
        except CalledProcessError:
            self._block_content.write(f"Cannot read block {self._inode}")

    def _update_inode_label(self) -> None:
        self._inode_label.update(f"Result for inode {self._inode}")

    def _update_block_content(self) -> None:
        self._block_content.clear()
        self._raw_block_content = decode_result(
            get_dd_output(self._partition, self._block_size, self._inode)
        )
        self._block_content.write(get_printable(self._raw_block_content))

    def compose(self) -> ComposeResult:
        yield Horizontal(self._inode_label, id="inode-label-container")
        yield Horizontal(self._block_content, id="block-content-container")
        yield Horizontal(
            self._block_count_label,
            id="block-count-container",
        )
        yield Horizontal(
            Button("Go back", id="go-back-button"), id="go-back-button-container"
        )
        yield Horizontal(
            Button("Previous", id="previous-button"),
            Button("Add block", id="add-block-button"),
            Button("Next", id="next-button"),
            id="block-buttons-container",
        )
        yield Horizontal(self._save_button, id="save-button-container")

    async def on_button_pressed(self, event: Button.Pressed) -> None:
        button_id = event.button.id
        if button_id == "go-back-button":
            self.app.pop_screen()
        elif button_id == "previous-button" and self._inode > 0:
            self._inode -= 1
            self._update()
        elif button_id == "add-block-button" and self._raw_block_content:
            self._saver.add(self._inode, self._raw_block_content)
            count = self._saver.get_selected_blocks_count()
            self._block_count_label.update(
                f"{count} block{'s' if count > 1 else ''} selected"
            )
            self._save_button.disabled = False
        elif button_id == "next-button":
            self._inode += 1
            self._update()
        elif button_id == "save-button":
            cast(SaveScreen, self.app.get_screen("save")).set_saver(self._saver)
            await self.app.push_screen("save")



"""First screen displayed to the user.
Allows user to enter search string and select partition."""

from textual._types import MessageTarget
from textual.app import ComposeResult
from textual.message import Message
from textual.screen import Screen
from textual.widgets import Button, Input, Label

from recoverpy.models.partition import Partition
from recoverpy.ui.widgets.partition_list import PartitionList


class ParamsScreen(Screen):
    _partition_list: PartitionList
    _search_input: Input
    _start_search_button: Button

    class Continue(Message):
        def __init__(
            self, sender: MessageTarget, searched_string: str, selected_partition: str
        ) -> None:
            self.searched_string = searched_string
            self.selected_partition = selected_partition
            super().__init__(sender)

    def compose(self) -> ComposeResult:
        self._partition_list = PartitionList()
        self._search_input = Input(name="search", placeholder="Search")
        self._start_search_button = Button(
            label="Start search", id="start-search-button", disabled=True
        )

        yield Label("Type a text to search for:")
        yield self._search_input
        yield Label("Available partitions:")
        yield self._partition_list
        yield self._start_search_button

    async def on_button_pressed(self) -> None:
        if self._partition_list.highlighted_child is None:
            return
        searched_string = self._search_input.value.strip()
        if len(searched_string) == 0:
            return
        selected_partition: Partition = self._partition_list.list_items[
            self._partition_list.highlighted_child.id
        ]
        await self.app.post_message(
            self.Continue(self, searched_string, selected_partition.get_full_name())
        )

    async def on_input_changed(self, event: Input.Changed) -> None:
        self._start_search_button.disabled = len(event.value.strip()) == 0



from subprocess import check_output
from typing import List, Tuple

from recoverpy.models.partition import Partition

_IGNORED_PARTITIONS_TYPES: Tuple[str, str] = (" loop ", "swap")


def get_partitions() -> List[Partition]:
    lsblk_output: str = _lsblk()
    return _format_lsblk_output(lsblk_output)


def _lsblk() -> str:
    return check_output(
        ["lsblk", "-r", "-n", "-o", "NAME,TYPE,FSTYPE,MOUNTPOINT"],
        encoding="utf-8",
    )


def _format_lsblk_output(lsblk_output: str) -> List[Partition]:
    partitions = []

    for line in lsblk_output.splitlines():
        if any(word in line for word in _IGNORED_PARTITIONS_TYPES):
            continue

        values = line.strip().split()

        if len(values) < 3:
            # Ignore if no FSTYPE detected
            continue

        if len(values) < 4:
            is_mounted = False
            mount_point = None
        else:
            is_mounted = True
            mount_point = values[3]

        partitions.append(
            Partition(
                name=values[0],
                fs_type=values[2],
                is_mounted=is_mounted,
                mount_point=mount_point,
            )
        )

    return partitions



"""A Textual ListView Widget for displaying partitions."""
from typing import Dict, Optional

from textual.widgets import Label, ListItem, ListView

from recoverpy.lib.lsblk import get_partitions
from recoverpy.models.partition import Partition


def _get_label(partition: Partition) -> Label:
    label_content = f"{partition.name} | {partition.fs_type}"
    if partition.is_mounted:
        label_content += f" | Mounted on: {partition.mount_point}"
    return Label(label_content)


class PartitionList(ListView):
    def __init__(self, *children, **kwargs) -> None:  # type: ignore
        super().__init__(*children, **kwargs)
        self.list_items: Dict[Optional[str], Partition] = {}
        self._append_partitions()

    def _append_partitions(self) -> None:
        for partition in get_partitions():
            list_item = ListItem(_get_label(partition), id=partition.name)
            if partition.is_mounted:
                list_item.add_class("mounted")
            self.list_items[list_item.id] = partition
            self.append(list_item)



from __future__ import annotations

from queue import Queue
from typing import List

from recoverpy.lib.helper import decode_result, get_dd_output, get_inode
from recoverpy.models.search_params import SearchParams


class ResultProcessor:
    def __init__(self, search_params: SearchParams):
        self.search_params = search_params

    def get_new_results(self, queue_object: Queue[bytes]) -> List[str]:
        """Consume grep output queue and format results."""

        queue_list: List[bytes] = list(queue_object.queue)
        queue_size = len(queue_list)
        queue_object.queue.clear()
        if queue_size == 0:
            return []

        decoded_results: List[str] = [decode_result(r) for r in queue_list]

        if not self.search_params.is_multi_line:
            return decoded_results

        final_results = [
            result for result in decoded_results if self.is_result_format_valid(result)
        ]
        return final_results

    def is_result_format_valid(self, result: str) -> bool:
        """Check if all searched lines are present in result block and next adjacent
        block, else result is a false positive and will not be displayed.
        """

        inode: int = int(get_inode(result))
        block_output: bytes = get_dd_output(
            self.search_params.partition,
            self.search_params.block_size * 8,
            int(inode / (self.search_params.block_size * 8)),
        )
        next_block_output: bytes = get_dd_output(
            self.search_params.partition,
            self.search_params.block_size * 8,
            int(inode / (self.search_params.block_size * 8) + 1),
        )
        both_block_output: str = decode_result(block_output) + decode_result(
            next_block_output
        )
        return all(
            line in both_block_output for line in self.search_params.searched_lines
        )

    def fix_line_start(self, line: str) -> str:
        result_index: int = line.find(self.search_params.searched_lines[0])
        return line[min(result_index, 15) :]

    def fix_inode(self, inode: int) -> int:
        """Fix result block number if search string is too far from beginning of
        returned inode number.
        """

        inode = int(inode / self.search_params.block_size)

        for _ in range(10):
            dd_output: str = decode_result(
                get_dd_output(
                    self.search_params.partition, self.search_params.block_size, inode
                )
            )
            if self.search_params.searched_lines[0] in dd_output:
                return inode
            inode += 1
        return inode



from os import geteuid
from re import findall
from subprocess import DEVNULL, call, check_output


def is_user_root() -> bool:
    return geteuid() == 0


def is_dependency_installed(command: str) -> bool:
    return call(["which", command], stdout=DEVNULL) == 0


def decode_result(result: bytes) -> str:
    return result.decode("utf-8", errors="ignore")


def get_printable(result: str) -> str:
    return "".join(([c for c in result.replace("\n", " ") if c.isprintable()]))


def get_block_size(partition: str) -> int:
    return int(
        check_output(
            ["blockdev", "--getbsz", partition],
            encoding="utf-8",
        )
    )


def get_inode(string: str) -> int:
    match = findall(r"^(\d+):", string)
    return int(match[0])


def get_dd_output(partition: str, block_size: int, inode: int) -> bytes:
    return check_output(
        [
            "dd",
            f"if={partition}",
            "count=1",
            "status=none",
            f"bs={block_size}",
            f"skip={inode}",
        ]
    )



from datetime import datetime
from pathlib import Path
from typing import Dict, List, Optional


class Saver:
    """Encapsulates all result saving related methods."""

    def __init__(self) -> None:
        self.save_path: Path = Path().absolute()
        self.last_saved_file: Optional[Path] = None
        self._results: Dict[int, str] = {}

    def add(self, inode: int, result: str) -> None:
        self._results[inode] = result

    def reset(self) -> None:
        self._results = {}

    def save(self) -> None:
        ordered_blocks: List[str] = [
            self._results[num] for num in sorted(self._results.keys())
        ]
        final_output: str = "\n".join(ordered_blocks)

        self._save_result_string(final_output)
        self.reset()

    def update_save_path(self, path: str) -> None:
        self.save_path = Path(path)

    def get_selected_blocks_count(self) -> int:
        return len(self._results)

    def _save_result_string(self, result: str) -> None:
        if self.save_path is None:
            return
        time_format: str = datetime.now().strftime("recoverpy-save-%Y-%m-%d-%H%M%S")
        file_name: Path = self.save_path / time_format

        self._write_to_file(file_name=file_name, content=result)

    def _write_to_file(self, file_name: Path, content: str) -> None:
        with open(file_name, "w") as save_file:
            save_file.write(content)

        self.last_saved_file = file_name



"""A Textual ListView widget consuming an asyncio Queue"""

from __future__ import annotations

from asyncio import Lock, Queue, sleep
from typing import List, cast

from textual.widget import Widget
from textual.widgets import Label, ListView

from recoverpy.models.grep_result import GrepResult


class GrepResultList(ListView):
    list_items_background_color = {0: "red", 1: "green"}

    def __init__(self, *children, **kwargs) -> None:  # type: ignore
        super().__init__(*children, **kwargs)
        self.results: List[str] = []
        self.lock = Lock()
        self.grep_results: List[GrepResult] = []

    async def start_consumer(self, queue: Queue[GrepResult]) -> None:
        while True:
            if not self._should_add_more():
                await sleep(0.1)
                continue
            grep_result = await queue.get()
            await self._append(grep_result)

    async def _append(self, grep_result: GrepResult) -> None:
        if grep_result.list_item is None:
            return
        async with self.lock:
            self.grep_results.append(grep_result)
            self._resize_item(len(self.grep_results) - 1, grep_result.list_item)
            await super().append(grep_result.list_item)

    def _get_list_index_to_show(self) -> int:
        return self.size.height + int(self.scroll_y) + 10

    def _should_add_more(self) -> bool:
        return len(self.children) < self._get_list_index_to_show()

    def on_resize(self) -> None:
        for index, item in enumerate(self.children):
            self._resize_item(index, item)

    def _resize_item(self, index: int, item: Widget) -> None:
        max_item_width = self.size.width - self.size.width // 20
        grep_result_line = self.grep_results[index].line

        cast(Label, item.children[0]).update(grep_result_line[:max_item_width])



"""Taken from official Textual official widget,
modified to allow directory selection."""

from __future__ import annotations

from dataclasses import dataclass
from pathlib import Path
from typing import Any, ClassVar

from rich.style import Style
from rich.text import Text, TextType
from textual.widgets import Tree
from textual.widgets._tree import TOGGLE_STYLE
from textual.widgets.tree import TreeNode

@dataclass
class DirEntry:
    """Attaches directory information ot a node."""

    path: str
    is_dir: bool
    loaded: bool = False


class DirectoryTree(Tree[DirEntry]):
    COMPONENT_CLASSES: ClassVar[set[str]] = {
        "tree--label",
        "tree--guides",
        "tree--guides-hover",
        "tree--guides-selected",
        "tree--cursor",
        "tree--highlight",
        "tree--highlight-line",
        "directory-tree--folder",
        "directory-tree--file",
        "directory-tree--extension",
        "directory-tree--hidden",
    }

    DEFAULT_CSS = """
    DirectoryTree > .directory-tree--folder {
        text-style: bold;
    }

    DirectoryTree > .directory-tree--file {

    }

    DirectoryTree > .directory-tree--extension {
        text-style: italic;
    }

    DirectoryTree > .directory-tree--hidden {
        color: $text 50%;
    }
    """

    def __init__(
        self,
        path: str,
        *,
        name: str | None = None,
        id: str | None = None,
        classes: str | None = None,
    ) -> None:
        self.path = path
        self.selected_dir = path
        super().__init__(
            path,
            data=DirEntry(path, True),
            name=name,
            id=id,
            classes=classes,
        )

    def process_label(self, label: TextType) -> Text:
        """Process a str or Text in to a label.
        Maybe overridden in a subclass to change modify how labels are rendered.

        Args:
            label (TextType): Label.

        Returns:
            Text: A Rich Text object.
        """
        if isinstance(label, str):
            text_label = Text(label)
        else:
            text_label = label
        first_line = text_label.split()[0]
        return first_line

    def render_label(
        self, node: TreeNode[DirEntry], base_style: Style, style: Style
    ) -> Text:
        node_label = node._label.copy()
        node_label.stylize(style)

        prefix = ("📂 " if node.is_expanded else "📁 ", base_style + TOGGLE_STYLE)
        node_label.stylize_before(
            self.get_component_rich_style("directory-tree--folder", partial=True)
        )

        if node_label.plain.startswith("."):
            node_label.stylize_before(
                self.get_component_rich_style("directory-tree--hidden")
            )

        text = Text.assemble(prefix, node_label)
        return text

    def load_directory(self, node: TreeNode[DirEntry]) -> None:
        assert node.data is not None
        dir_path = Path(node.data.path)
        node.data.loaded = True
        directory = sorted(
            list(dir_path.iterdir()),
            key=lambda path: (not path.is_dir(), path.name.lower()),
        )
        for path in directory:
            if not path.is_dir():
                continue
            node.add(
                path.name,
                data=DirEntry(str(path), path.is_dir()),
                allow_expand=path.is_dir(),
            )
        node.expand()

    def on_mount(self) -> None:
        self.load_directory(self.root)

    def on_tree_node_expanded(self, event: Tree.NodeSelected[Any]) -> None:
        event.stop()
        dir_entry = event.node.data
        if dir_entry is None:
            return
        if dir_entry.is_dir:
            if not dir_entry.loaded:
                self.load_directory(event.node)

    def on_tree_node_selected(self, event: Tree.NodeSelected[Any]) -> None:
        event.stop()
        dir_entry = event.node.data
        if dir_entry is None:
            return
        if dir_entry.is_dir:
            self.selected_dir = dir_entry.path
            print("Selected dir:", self.selected_dir)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['recoverpy',
 'recoverpy.lib',
 'recoverpy.lib.search',
 'recoverpy.models',
 'recoverpy.ui',
 'recoverpy.ui.css',
 'recoverpy.ui.screens',
 'recoverpy.ui.widgets']

package_data = \
{'': ['*']}

install_requires = \
['textual>=0.9,<0.12']

setup_kwargs = {
    'name': 'recoverpy',
    'version': '2.0.1',
    'description': 'A TUI to recover overwritten or deleted data.',
    'long_description': '<div align="center">\n    <img src="docs/assets/logo.png" alt="RecoverPy">\n</div>\n\n<p align="center">\n    <em>Recover overwritten or deleted data.</em>\n</p>\n\n<p align="center">\n<a href="https://img.shields.io/github/v/release/pablolec/recoverpy" target="_blank">\n    <img src="https://img.shields.io/github/v/release/pablolec/recoverpy" alt="Release">\n</a>\n<a href="https://github.com/PabloLec/recoverpy/blob/main/LICENSE" target="_blank">\n    <img src="https://img.shields.io/github/license/pablolec/recoverpy" alt="License">\n</a>\n<a href="https://pepy.tech/project/recoverpy" target="_blank">\n    <img src="https://static.pepy.tech/personalized-badge/recoverpy?period=total&units=abbreviation&left_color=grey&right_color=red&left_text=downloads" alt="Downloads">\n</a>\n\n<a href="#" target="_blank">\n    <img src="https://github.com/PabloLec/recoverpy/actions/workflows/pytest.yml/badge.svg?branch=main" alt="Tests">\n</a>\n</p>\n\n---\n\n<!--ts-->\n   * [Demo](#Demo)\n   * [Installation](#Installation)\n      * [Dependancies](#arrow_right-dependancies)\n      * [Installation from pip](#arrow_right-installation-from-pip)\n   * [Usage](#Usage)\n   * [Tips](#Tips)\n   * [Contributing](#Contributing)\n<!--te-->\n\n---\n\n# RecoverPy\n\nRecoverPy is a powerful tool that leverages your system capabilities to recover lost files.\n\nUnlike others, you can not only recover deleted files but also **overwritten** data.\n\nEvery block of your partition will be scanned. You can even find a string in binary files.\n## Demo\n\n<p align="center">\n    <img src="docs/assets/demo.gif">\n</p>\n\n## Installation\n\n:penguin: RecoverPy is currently only available on Linux systems.\n\n#### :arrow_right: Dependancies\n\n**Mandatory:** To list and search through your partitions, recoverpy uses `grep`, `dd`, and `lsblk` commands. Although, if you\'re running a major Linux distrucition these tools should already be installed.\n\n**Optional:** To display real time grep progress, you can install `progress`.\n\nTo install all dependencies:\n- Debian-like: `apt install grep coreutils util-linux progress`\n- Arch: `pacman -S grep coreutils util-linux progress`\n- Fedora: `dnf install grep coreutils util-linux progress`\n\n#### :arrow_right: Installation from pip\n\n`python3 -m pip install recoverpy`\n\n## Usage\n\n```bash\npython3 -m recoverpy\n```\n\n:red_circle: **You must be root or use sudo**.\n\n---\n\n:one: **Select the system partition** in which your file was. If you are out of luck, you can alternatively search in your home partition, maybe your IDE, text editor, etc. made a backup at some point.\n\n:two: **Type a text string to search**. See tips below for better results.\n\n:three: **Start search**, Results will appear in the left-hand box.\n\n:four: **Select a result**.\n\n:five: Once you have found your precious, **select `Open`**.\n\n:six: You can now either save this block individually or explore neighboring blocks for the remaining parts of the file. You could then save it all in one file.\n\n## Tips\n\n- Always do backups! Yes, maybe too late...\n- **Unmount your partition before you do anything!** Although you can search with your partition still mounted, it is highly recommended to unmount your partition to avoid any alteration to your file.\n\nRegarding the searched string:\n\n- Be concise, find something that could be unique to your file.\n- Stay simple, your string is escaped but exotic characters may affect your results.\n- Try to remember the last edit you have made to your file.\n\nWhen you have found your file:\n\n- You might see multiple results. Your system often use different partion blocks to save successive versions of a file. Make sure you\'ve found the last version.\n- Try exploring neighboring blocks to be sure to save your whole file.\n\n## Contributing\n\nThank you for considering contributing to RecoverPy.\nAny request, bug report or PR are welcome. Please read the [contributing guide](CONTRIBUTING.md).\n',
    'author': 'PabloLec',
    'author_email': 'pablolec@pm.me',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/PabloLec/recoverpy',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8.1,<4.0.0',
}


setup(**setup_kwargs)



"""Screen used to display a modal dialog."""
from typing import Callable, Optional

from textual.app import ComposeResult
from textual.containers import Grid, Horizontal
from textual.screen import Screen
from textual.widgets import Button, Label


class ModalScreen(Screen):
    _message_label = Label("", id="modal-message")
    _callback: Callable  # type: ignore

    def set(self, message: str, callback: Optional[Callable] = None) -> None:  # type: ignore
        self._message_label.update(message)
        self._callback = callback or self.app.pop_screen

    def compose(self) -> ComposeResult:
        yield Grid(
            self._message_label,
            Horizontal(
                Button("OK", id="ok-button"),
                id="modal-button-container",
            ),
            id="modal-container",
        )

    def on_button_pressed(self) -> None:
        self._callback()



"""Screen used to confirm save."""
from typing import Optional

from textual._types import MessageTarget
from textual.app import ComposeResult
from textual.containers import Horizontal, Vertical
from textual.message import Message
from textual.screen import Screen
from textual.widgets import Button, Label

from recoverpy.lib.saver import Saver
from recoverpy.ui.screens.screen_path_edit import PathEditScreen


class SaveScreen(Screen):
    class Saved(Message):
        def __init__(self, sender: MessageTarget, save_path: str) -> None:
            self.save_path = save_path
            super().__init__(sender)

    def __init__(self, *args, **kwargs) -> None:  # type: ignore
        self._saver: Optional[Saver] = None
        self._save_path_label = Label("", id="save-path-label")
        super().__init__(*args, **kwargs)

    def set_saver(self, saver: Saver) -> None:
        self._saver = saver
        self._set_save_path()

    def compose(self) -> ComposeResult:
        yield Vertical(
            Label("Current save path:", id="save-path-title-label"),
            self._save_path_label,
            id="save-path-container",
        )
        yield Horizontal(
            Button("Go back", id="go-back-button"),
            Button("Edit save path", id="edit-save-path-button"),
            Button("Save", id="save-button"),
            id="action-buttons-container",
        )

    def _set_save_path(self) -> None:
        if self._saver:
            self._save_path_label.update(str(self._saver.save_path))

    async def on_button_pressed(self, event: Button.Pressed) -> None:
        button_id = event.button.id
        if button_id == "go-back-button":
            self.app.pop_screen()
        elif button_id == "edit-save-path-button":
            await self.app.push_screen("path_edit")
        elif button_id == "save-button" and self._saver:
            self._saver.save()
            self.app.pop_screen()
            if self._saver.last_saved_file:
                await self.app.post_message(
                    self.Saved(
                        self, str(self._saver.save_path / self._saver.last_saved_file)
                    )
                )

    async def on_path_edit_screen_confirm(self, event: PathEditScreen.Confirm) -> None:
        if self._saver:
            self._saver.update_save_path(event.selected_dir)
            self._set_save_path()
